@@ -295,6 +295,7 @@ _SPIRV_OP(TypePipeStorage, 322)
 _SPIRV_OP(ConstantPipeStorage, 323)
 _SPIRV_OP(CreatePipeFromPipeStorage, 324)
 _SPIRV_OP(ModuleProcessed, 330)
+_SPIRV_OP(ExecutionModeId, 331)
 _SPIRV_OP(DecorateId, 332)
 _SPIRV_OP(GroupNonUniformElect, 333)
 _SPIRV_OP(GroupNonUniformAll, 334)
@@ -335,6 +336,12 @@ _SPIRV_OP(SUDotKHR, 4452)
 _SPIRV_OP(SDotAccSatKHR, 4453)
 _SPIRV_OP(UDotAccSatKHR, 4454)
 _SPIRV_OP(SUDotAccSatKHR, 4455)
+_SPIRV_OP(TypeCooperativeMatrixKHR, 4456)
+_SPIRV_OP(CooperativeMatrixLoadKHR, 4457)
+_SPIRV_OP(CooperativeMatrixStoreKHR, 4458)
+_SPIRV_OP(CooperativeMatrixMulAddKHR, 4459)
+_SPIRV_OP(CooperativeMatrixLengthKHR, 4460)
+_SPIRV_OP(ReadClockKHR, 5056)
 _SPIRV_OP(SubgroupShuffleINTEL, 5571)
 _SPIRV_OP(SubgroupShuffleDownINTEL, 5572)
 _SPIRV_OP(SubgroupShuffleUpINTEL, 5573)
